from coin_sdk.number_portability.domain import MessageType
from coin_sdk.number_portability.messages.common import EnumRepeatsBuilder
from coin_sdk.number_portability.messages.common import HeaderCreator
from coin_sdk.number_portability.messages.message import Message
from .porting_request_answer import PortingRequestAnswer
from .porting_request_answer_seq import PortingRequestAnswerSeq
from .porting_request_answer_repeats import PortingRequestAnswerRepeats
from .porting_request_answer_message import PortingRequestAnswerMessage
from .porting_request_answer_body import PortingRequestAnswerBody


class PortingRequestAnswerBuilder(HeaderCreator):
    def __init__(self):
        super().__init__()
        self._blocking = None
        self._dossierid = None
        self._answer_seqs = []

    def set_dossierid(self, dossierid: str):
        self._dossierid = dossierid
        return self

    def set_blocking(self, blocking: str):
        self._blocking = blocking
        return self

    def add_porting_request_answer_seq(self):
        return PortingRequestAnswerRepeatsBuilder(self)

    def build(self):
        repeats = [PortingRequestAnswerRepeats(seq) for seq in self._answer_seqs]
        answer = PortingRequestAnswer(
            self._dossierid,
            self._blocking,
            repeats
        )
        body = PortingRequestAnswerBody(answer)
        message = PortingRequestAnswerMessage(self._header, body)
        return Message(message, MessageType.PORTING_REQUEST_ANSWER_V1)


class PortingRequestAnswerRepeatsBuilder(EnumRepeatsBuilder):
    def __init__(self, answer_builder: PortingRequestAnswerBuilder):
        super().__init__()
        self._answer_builder = answer_builder
        self._blockingcode = None
        self._firstpossibledate = None
        self._note = None
        self._donornetworkoperator = None
        self._donorserviceprovider = None

    def set_number_series(self, start: str, end: str):
        self._set_number_series(start, end)
        return self

    def set_blockingcode(self, blockingcode: str):
        self._blockingcode = blockingcode
        return self

    def set_firstpossibledate(self, firstpossibledate: str):
        self._firstpossibledate = firstpossibledate
        return self

    def set_note(self, note: str):
        self._note = note
        return self

    def set_donornetworkoperator(self, donornetworkoperator: str):
        self._donornetworkoperator = donornetworkoperator
        return self

    def set_donorserviceprovider(self, donorserviceprovider):
        self._donorserviceprovider = donorserviceprovider
        return self

    def finish(self):
        seq = PortingRequestAnswerSeq(self._number_series, self._blockingcode, self._firstpossibledate, self._note, self._donornetworkoperator, self._donorserviceprovider)
        self._answer_builder._answer_seqs.append(seq)
        return self._answer_builder



import logging
import time
from abc import abstractmethod, ABC
from json import JSONDecodeError
from typing import Type, Optional, Callable

import sseclient
from requests import ConnectionError
from requests.exceptions import ChunkedEncodingError

from coin_sdk.number_portability.domain import MessageType, ConfirmationStatus
from coin_sdk.common.securityservice import SecurityService
from coin_sdk.number_portability.utils import json2obj, handle_http_error, get_stream
from coin_sdk.number_portability.npconfig import NpConfig

logger = logging.getLogger(__name__)


class OffsetPersister(ABC):
    @abstractmethod
    def persist_offset(self, offset):
        pass

    @abstractmethod
    def get_persisted_offset(self):
        pass


class Receiver(ABC):
    class RetriesLeft:
        def __init__(self, number_of_retries):
            self._default = number_of_retries
            self._number_of_retries = number_of_retries

        def reset(self):
            self._number_of_retries = self._default

        def decrement(self):
            self._number_of_retries -= 1

        def get(self):
            return self._number_of_retries

        def __call__(self):
            return self._number_of_retries > 0

    def __init__(self, config: NpConfig):
        self._security_service = SecurityService(config)
        self._config = config
        self._offset_persister: Optional[OffsetPersister] = None
        self._recover_offset: Callable[[int], int] = lambda x: x
        self._event_map = {
            MessageType.PORTING_REQUEST_V1.get_event_type(): self.on_porting_request,
            MessageType.PORTING_REQUEST_ANSWER_V1.get_event_type(): self.on_porting_request_answer,
            MessageType.PORTING_REQUEST_ANSWER_DELAYED_V1.get_event_type(): self.on_porting_request_answer_delayed,
            MessageType.PORTING_PERFORMED_V1.get_event_type(): self.on_porting_performed,
            MessageType.DEACTIVATION_V1.get_event_type(): self.on_deactivation,
            MessageType.CANCEL_V1.get_event_type(): self.on_cancel,
            MessageType.ERROR_FOUND_V1.get_event_type(): self.on_error_found
        }

    def start_stream(
            self,
            offset: int = None,
            confirmation_status: ConfirmationStatus = None,
            offset_persister: Type[OffsetPersister] = None,
            recover_offset: Callable[[int], int] = lambda x: x,
            message_types: [MessageType] = None
    ):
        self._running = True
        self._setup(confirmation_status, offset_persister, recover_offset)
        retries_left = self.RetriesLeft(self._config.number_of_retries)
        while retries_left():
            try:
                self._connect(offset, confirmation_status, message_types, retries_left)
                return
            except (ConnectionError, ChunkedEncodingError) as e:
                # ChunckedEncodingError occurs when backend stops while waiting for new events
                logger.error(type(e).__name__)
                logger.error(e)
                logger.error(f'Trying to reconnect in {self._config.backoff_period} seconds. Retries left: {retries_left.get()}')
                retries_left.decrement()
                if self._offset_persister:
                    offset = self._offset_persister.get_persisted_offset()
                    offset = self._recover_offset(offset)
                time.sleep(self._config.backoff_period)

    def _setup(
            self,
            confirmation_status: ConfirmationStatus,
            offset_persister: Type[OffsetPersister],
            recover_offset: Callable[[int], int]
    ):
        if confirmation_status == ConfirmationStatus.ALL and not offset_persister:
            raise ValueError('offset_persister should be given when confirmation_status equals ALL')
        if offset_persister and not issubclass(offset_persister, OffsetPersister):
            raise ValueError(f'offset_persister should be a subclass of {OffsetPersister.__module__}.OffsetPersister')
        self._offset_persister = offset_persister and offset_persister()
        self._recover_offset = recover_offset

    def _connect(self, offset: int, confirmation_status: ConfirmationStatus, message_types: [MessageType], retries_left: RetriesLeft):
        logger.debug('Opening stream')
        response = get_stream(self._config.sse_url, offset, confirmation_status, message_types, self._security_service)
        logger.debug(f'url: {response.request.url}')
        handle_http_error(response)
        client = sseclient.SSEClient(response)
        retries_left.reset()
        self._consume_stream(client)

    def _consume_stream(self, client: sseclient.SSEClient):
        for event in client.events():
            if self._running == False:
                return
            logger.debug('Received event')
            logger.debug(f'{event}')
            if event.data:
                self._process_event(event)
            else:
                self.on_keep_alive(event.id)

    def stop(self):
        self._running = False

    def _process_event(self, event):
        try:
            event_type = event.event.lower()
            logger.debug(f'Event: {event.event}')
            message = json2obj(event.data).message
            logger.debug(f'Message: {message}')
            message_id = event.id
            logger.debug(f'Message id: {message_id}')
            event_handler = self._event_map.get(event_type, None)
            if event_handler:
                event_handler(message_id, message)
                self._persist_offset(message_id)
            else:
                logger.error(f"Number Portability Message with the following content isn't supported: {event}")
        except (JSONDecodeError, AttributeError):
            logger.error(f"Conversion of Number Portability Message failed for the following event: {event}")

    def _persist_offset(self, message_id):
        if self._offset_persister:
            self._offset_persister.persist_offset(message_id)

    @abstractmethod
    def on_keep_alive(self, message_id):
        pass

    @abstractmethod
    def on_porting_request(self, message_id, message):
        pass

    @abstractmethod
    def on_porting_request_answer(self, message_id, message):
        pass

    @abstractmethod
    def on_porting_request_answer_delayed(self, message_id, message):
        pass

    @abstractmethod
    def on_porting_performed(self, message_id, message):
        pass

    @abstractmethod
    def on_deactivation(self, message_id, message):
        pass

    @abstractmethod
    def on_cancel(self, message_id, message):
        pass

    @abstractmethod
    def on_error_found(self, message_id, message):
        pass



import logging

import requests

from coin_sdk.number_portability.npconfig import NpConfig

from coin_sdk.common.sendrequest import send_request
from coin_sdk.number_portability.domain import MessageType
from coin_sdk.number_portability.messages.message_response import MessageResponse
from coin_sdk.common.securityservice import SecurityService

logger = logging.getLogger(__name__)


def _send_request(request, url, json, security_service):
    response_json = send_request(request, url, json, security_service)
    try:
        return MessageResponse(response_json.transaction_id)
    except AttributeError:
        return response_json


class Sender:
    def __init__(self, config: NpConfig):
        self._security_service = SecurityService(config)
        self._config = config

    def send_message(self, message):
        logger.info(f'Sending message: {message}')
        message_type = message.get_message_type()
        message_dict = message.to_dict()
        url = f'{self._config.api_url}/{message_type.value}'
        logger.debug(f'url: {url}')
        return _send_request(requests.post, url, message_dict, self._security_service)

    def confirm(self, transaction_id):
        logger.info(f'Sending conformation for id: {transaction_id}')
        url = f'{self._config.api_url}/{MessageType.CONFIRMATION_V1.value}/{transaction_id}'
        logger.debug(f'url: {url}')
        json = {'transactionId': transaction_id}
        return _send_request(requests.put, url, json, self._security_service)



# coding: utf-8

"""
    Coin CRDB Rest API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 3.0.0
    Contact: devops@coin.nl
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class PortingRequestAnswerDelayed(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dossierid': 'str',
        'donornetworkoperator': 'str',
        'donorserviceprovider': 'str',
        'answerduedatetime': 'str',
        'reasoncode': 'str'
    }

    attribute_map = {
        'dossierid': 'dossierid',
        'donornetworkoperator': 'donornetworkoperator',
        'donorserviceprovider': 'donorserviceprovider',
        'answerduedatetime': 'answerduedatetime',
        'reasoncode': 'reasoncode'
    }

    def __init__(self, dossierid=None, donornetworkoperator=None, donorserviceprovider=None, answerduedatetime=None, reasoncode=None):  # noqa: E501
        """PortingRequestAnswerDelayed - a model defined in Swagger"""  # noqa: E501

        self._dossierid = None
        self._donornetworkoperator = None
        self._donorserviceprovider = None
        self._answerduedatetime = None
        self._reasoncode = None
        self.discriminator = None

        self.dossierid = dossierid
        self.donornetworkoperator = donornetworkoperator
        if donorserviceprovider is not None:
            self.donorserviceprovider = donorserviceprovider
        if answerduedatetime is not None:
            self.answerduedatetime = answerduedatetime
        if reasoncode is not None:
            self.reasoncode = reasoncode

    @property
    def dossierid(self):
        """Gets the dossierid of this PortingRequestAnswerDelayed.  # noqa: E501


        :return: The dossierid of this PortingRequestAnswerDelayed.  # noqa: E501
        :rtype: str
        """
        return self._dossierid

    @dossierid.setter
    def dossierid(self, dossierid):
        """Sets the dossierid of this PortingRequestAnswerDelayed.


        :param dossierid: The dossierid of this PortingRequestAnswerDelayed.  # noqa: E501
        :type: str
        """
        if dossierid is None:
            raise ValueError("Invalid value for `dossierid`, must not be `None`")  # noqa: E501
        if dossierid is not None and not re.search(r'[-0-9A-Z]{5,26}', dossierid):  # noqa: E501
            raise ValueError(r"Invalid value for `dossierid`, must be a follow pattern or equal to `/[-0-9A-Z]{5,26}/`")  # noqa: E501

        self._dossierid = dossierid

    @property
    def donornetworkoperator(self):
        """Gets the donornetworkoperator of this PortingRequestAnswerDelayed.  # noqa: E501


        :return: The donornetworkoperator of this PortingRequestAnswerDelayed.  # noqa: E501
        :rtype: str
        """
        return self._donornetworkoperator

    @donornetworkoperator.setter
    def donornetworkoperator(self, donornetworkoperator):
        """Sets the donornetworkoperator of this PortingRequestAnswerDelayed.


        :param donornetworkoperator: The donornetworkoperator of this PortingRequestAnswerDelayed.  # noqa: E501
        :type: str
        """
        if donornetworkoperator is None:
            raise ValueError("Invalid value for `donornetworkoperator`, must not be `None`")  # noqa: E501
        if donornetworkoperator is not None and not re.search(r'[0-9A-Z]{3,6}', donornetworkoperator):  # noqa: E501
            raise ValueError(r"Invalid value for `donornetworkoperator`, must be a follow pattern or equal to `/[0-9A-Z]{3,6}/`")  # noqa: E501

        self._donornetworkoperator = donornetworkoperator

    @property
    def donorserviceprovider(self):
        """Gets the donorserviceprovider of this PortingRequestAnswerDelayed.  # noqa: E501


        :return: The donorserviceprovider of this PortingRequestAnswerDelayed.  # noqa: E501
        :rtype: str
        """
        return self._donorserviceprovider

    @donorserviceprovider.setter
    def donorserviceprovider(self, donorserviceprovider):
        """Sets the donorserviceprovider of this PortingRequestAnswerDelayed.


        :param donorserviceprovider: The donorserviceprovider of this PortingRequestAnswerDelayed.  # noqa: E501
        :type: str
        """
        if donorserviceprovider is not None and not re.search(r'[0-9A-Z]{3,6}', donorserviceprovider):  # noqa: E501
            raise ValueError(r"Invalid value for `donorserviceprovider`, must be a follow pattern or equal to `/[0-9A-Z]{3,6}/`")  # noqa: E501

        self._donorserviceprovider = donorserviceprovider

    @property
    def answerduedatetime(self):
        """Gets the answerduedatetime of this PortingRequestAnswerDelayed.  # noqa: E501


        :return: The answerduedatetime of this PortingRequestAnswerDelayed.  # noqa: E501
        :rtype: str
        """
        return self._answerduedatetime

    @answerduedatetime.setter
    def answerduedatetime(self, answerduedatetime):
        """Sets the answerduedatetime of this PortingRequestAnswerDelayed.


        :param answerduedatetime: The answerduedatetime of this PortingRequestAnswerDelayed.  # noqa: E501
        :type: str
        """
        if answerduedatetime is not None and not re.search(r'[0-9]{14}', answerduedatetime):  # noqa: E501
            raise ValueError(r"Invalid value for `answerduedatetime`, must be a follow pattern or equal to `/[0-9]{14}/`")  # noqa: E501

        self._answerduedatetime = answerduedatetime

    @property
    def reasoncode(self):
        """Gets the reasoncode of this PortingRequestAnswerDelayed.  # noqa: E501


        :return: The reasoncode of this PortingRequestAnswerDelayed.  # noqa: E501
        :rtype: str
        """
        return self._reasoncode

    @reasoncode.setter
    def reasoncode(self, reasoncode):
        """Sets the reasoncode of this PortingRequestAnswerDelayed.


        :param reasoncode: The reasoncode of this PortingRequestAnswerDelayed.  # noqa: E501
        :type: str
        """
        if reasoncode is not None and not re.search(r'[1-9][0-9]{0,4}', reasoncode):  # noqa: E501
            raise ValueError(r"Invalid value for `reasoncode`, must be a follow pattern or equal to `/[1-9][0-9]{0,4}/`")  # noqa: E501

        self._reasoncode = reasoncode

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PortingRequestAnswerDelayed, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PortingRequestAnswerDelayed):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



class ErrorMessage:
    def __init__(self, transaction_id, errors):
        self.transaction_id = transaction_id
        self.errors = errors

    def __str__(self):
        return '\n'.join([f'{err.code}: {err.message}' for err in self.errors])



from enum import Enum


class ConfirmationStatus(Enum):
    CONFIRMED = "Confirmed"
    ALL = "All"


class MessageType(Enum):
    PORTING_REQUEST_V1 = "portingrequest"
    PORTING_REQUEST_ANSWER_V1 = "portingrequestanswer"
    PORTING_PERFORMED_V1 = "portingperformed"
    PORTING_REQUEST_ANSWER_DELAYED_V1 = "pradelayed"
    CANCEL_V1 = "cancel"
    DEACTIVATION_V1 = "deactivation"
    CONFIRMATION_V1 = "confirmations"
    ERROR_FOUND_V1 = "errorfound"
    _VERSION_SUFFIX_V1 = "-v1"

    def get_event_type(self):
        return f'{self.value}{self._VERSION_SUFFIX_V1.value}'



import logging

from coin_sdk.number_portability.utils import json2obj, handle_http_error

logger = logging.getLogger(__name__)


def send_request(request, url, json, security_service):
    method = request.__name__
    headers = security_service.generate_headers(url, method, json)
    cookie = security_service.generate_jwt()
    logger.debug(f'Making request: {method}')
    response = request(url, json=json, headers=headers, cookies=cookie)
    logger.debug(f'Header: {response.request.headers}')
    logger.debug(f'Body: {response.request.body}')
    handle_http_error(response)
    logger.debug('Converting JSON response to Python')
    response_json = json2obj(response.text)
    logger.debug(f'Response: {response_json}')
    return response_json



import json
import logging
from collections import namedtuple
from http import HTTPStatus
from json import JSONDecodeError

import requests

from coin_sdk.number_portability.domain import ConfirmationStatus, MessageType
from coin_sdk.number_portability.messages.common import ErrorMessage
from coin_sdk.common.securityservice import SecurityService

logger = logging.getLogger(__name__)


def _json_object_hook(d):
    return namedtuple('np', d.keys())(*d.values())


def json2obj(data):
    try:
        return json.loads(data, object_hook=_json_object_hook)
    except JSONDecodeError:
        return data


def handle_http_error(response):
    logger.debug('Checking for errors')
    status = response.status_code
    logger.debug(f'Http Status: {status}')
    if status == HTTPStatus.OK:
        return
    logger.error(f'Error: {response.text}')
    description = HTTPStatus(status).description
    try:
        error_message = json2obj(response.text)
        error_object = ErrorMessage(error_message.transactionId, error_message.errors)
        raise requests.HTTPError(f'HTTP Status: {status}, {description}\n{str(error_object)}', response=error_object)
    except AttributeError:
        logger.error(response)
        raise requests.HTTPError(f'HTTP Status: {status}, {description}', response=response)


def get_stream(url: str, offset: int, confirmation_status: ConfirmationStatus, message_types: [MessageType], security_service: SecurityService):
    params = {
        'offset': offset,
        'messageTypes': message_types and ','.join([message_type.value for message_type in message_types]),
        'confirmationStatus': confirmation_status and confirmation_status.value
    }
    logger.debug(f'Request parameters: {params}')
    headers = security_service.generate_headers(url)
    cookie = security_service.generate_jwt()
    return requests.get(url, stream=True, headers=headers, cookies=cookie, params=params)



from . import EnumRepeats
from . import EnumProfileSeq
from . import NumberSeries


class EnumRepeatsBuilder:
    def __init__(self):
        self._enum_repeats = []
        self._number_series = None

    def _add_enum_prof_seq(self, *profileids):
        self._enum_repeats = [EnumRepeats(EnumProfileSeq(profileid)) for profileid in profileids]

    def _set_number_series(self, start: str, end: str):
        self._number_series = NumberSeries(start, end)



# coding: utf-8

"""
    Coin CRDB Rest API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 3.0.0
    Contact: devops@coin.nl
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from coin_sdk.number_portability.messages.portingrequestanswer.porting_request_answer_repeats import PortingRequestAnswerRepeats  # noqa: F401,E501


class PortingRequestAnswer(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dossierid': 'str',
        'blocking': 'str',
        'repeats': 'list[PortingRequestAnswerRepeats]'
    }

    attribute_map = {
        'dossierid': 'dossierid',
        'blocking': 'blocking',
        'repeats': 'repeats'
    }

    def __init__(self, dossierid=None, blocking=None, repeats=None):  # noqa: E501
        """PortingRequestAnswer - a model defined in Swagger"""  # noqa: E501

        self._dossierid = None
        self._blocking = None
        self._repeats = None
        self.discriminator = None

        self.dossierid = dossierid
        self.blocking = blocking
        self.repeats = repeats

    @property
    def dossierid(self):
        """Gets the dossierid of this PortingRequestAnswer.  # noqa: E501


        :return: The dossierid of this PortingRequestAnswer.  # noqa: E501
        :rtype: str
        """
        return self._dossierid

    @dossierid.setter
    def dossierid(self, dossierid):
        """Sets the dossierid of this PortingRequestAnswer.


        :param dossierid: The dossierid of this PortingRequestAnswer.  # noqa: E501
        :type: str
        """
        if dossierid is None:
            raise ValueError("Invalid value for `dossierid`, must not be `None`")  # noqa: E501
        if dossierid is not None and not re.search(r'[-0-9A-Z]{5,26}', dossierid):  # noqa: E501
            raise ValueError(r"Invalid value for `dossierid`, must be a follow pattern or equal to `/[-0-9A-Z]{5,26}/`")  # noqa: E501

        self._dossierid = dossierid

    @property
    def blocking(self):
        """Gets the blocking of this PortingRequestAnswer.  # noqa: E501


        :return: The blocking of this PortingRequestAnswer.  # noqa: E501
        :rtype: str
        """
        return self._blocking

    @blocking.setter
    def blocking(self, blocking):
        """Sets the blocking of this PortingRequestAnswer.


        :param blocking: The blocking of this PortingRequestAnswer.  # noqa: E501
        :type: str
        """
        if blocking is None:
            raise ValueError("Invalid value for `blocking`, must not be `None`")  # noqa: E501
        if blocking is not None and not re.search(r'[Y|N]', blocking):  # noqa: E501
            raise ValueError(r"Invalid value for `blocking`, must be a follow pattern or equal to `/[Y|N]/`")  # noqa: E501

        self._blocking = blocking

    @property
    def repeats(self):
        """Gets the repeats of this PortingRequestAnswer.  # noqa: E501


        :return: The repeats of this PortingRequestAnswer.  # noqa: E501
        :rtype: list[PortingRequestAnswerRepeats]
        """
        return self._repeats

    @repeats.setter
    def repeats(self, repeats):
        """Sets the repeats of this PortingRequestAnswer.


        :param repeats: The repeats of this PortingRequestAnswer.  # noqa: E501
        :type: list[PortingRequestAnswerRepeats]
        """
        if repeats is None:
            raise ValueError("Invalid value for `repeats`, must not be `None`")  # noqa: E501

        self._repeats = repeats

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PortingRequestAnswer, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PortingRequestAnswer):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from coin_sdk.number_portability.domain import MessageType
from coin_sdk.number_portability.messages.common import EnumRepeatsBuilder
from coin_sdk.number_portability.messages.common import HeaderCreator
from coin_sdk.number_portability.messages.message import Message
from .deactivation import Deactivation
from .deactivation_seq import DeactivationSeq
from .deactivation_repeats import  DeactivationRepeats
from .deactivation_body import DeactivationBody
from .deactivation_message import  DeactivationMessage


class DeactivationBuilder(HeaderCreator):
    def __init__(self):
        super().__init__()
        self._currentnetworkoperator = None
        self._originalnetworkoperator = None
        self._dossierid = None
        self._deactivation_seqs = []

    def set_dossierid(self, dossierid: str):
        self._dossierid = dossierid
        return self

    def set_currentnetworkoperator(self, currentnetworkoperator: str):
        self._currentnetworkoperator = currentnetworkoperator
        return self

    def set_originalnetworkoperator(self, originalnetworkoperator: str):
        self._originalnetworkoperator = originalnetworkoperator
        return self

    def add_deactivation_seq(self):
        return DeactivationRepeatsBuilder(self)

    def build(self):
        repeats = [DeactivationRepeats(seq) for seq in self._deactivation_seqs]
        deactivation = Deactivation(
            self._dossierid,
            self._currentnetworkoperator,
            self._originalnetworkoperator,
            repeats
        )
        body = DeactivationBody(deactivation)
        message = DeactivationMessage(self._header, body)
        return Message(message, MessageType.DEACTIVATION_V1)


class DeactivationRepeatsBuilder(EnumRepeatsBuilder):
    def __init__(self, deactivation_builder: DeactivationBuilder):
        super().__init__()
        self._deactivation_builder = deactivation_builder

    def set_number_series(self, start: str, end: str):
        self._set_number_series(start, end)
        return self

    def finish(self):
        seq = DeactivationSeq(self._number_series)
        self._deactivation_builder._deactivation_seqs.append(seq)
        return self._deactivation_builder



# coding: utf-8

"""
    Coin CRDB Rest API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 3.0.0
    Contact: devops@coin.nl
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from coin_sdk.number_portability.messages.portingperformed.porting_performed_repeats import PortingPerformedRepeats  # noqa: F401,E501


class PortingPerformed(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dossierid': 'str',
        'recipientnetworkoperator': 'str',
        'donornetworkoperator': 'str',
        'actualdatetime': 'str',
        'batchporting': 'str',
        'repeats': 'list[PortingPerformedRepeats]'
    }

    attribute_map = {
        'dossierid': 'dossierid',
        'recipientnetworkoperator': 'recipientnetworkoperator',
        'donornetworkoperator': 'donornetworkoperator',
        'actualdatetime': 'actualdatetime',
        'batchporting': 'batchporting',
        'repeats': 'repeats'
    }

    def __init__(self, dossierid=None, recipientnetworkoperator=None, donornetworkoperator=None, actualdatetime=None, batchporting=None, repeats=None):  # noqa: E501
        """PortingPerformed - a model defined in Swagger"""  # noqa: E501

        self._dossierid = None
        self._recipientnetworkoperator = None
        self._donornetworkoperator = None
        self._actualdatetime = None
        self._batchporting = None
        self._repeats = None
        self.discriminator = None

        self.dossierid = dossierid
        self.recipientnetworkoperator = recipientnetworkoperator
        self.donornetworkoperator = donornetworkoperator
        if actualdatetime is not None:
            self.actualdatetime = actualdatetime
        if batchporting is not None:
            self.batchporting = batchporting
        self.repeats = repeats

    @property
    def dossierid(self):
        """Gets the dossierid of this PortingPerformed.  # noqa: E501


        :return: The dossierid of this PortingPerformed.  # noqa: E501
        :rtype: str
        """
        return self._dossierid

    @dossierid.setter
    def dossierid(self, dossierid):
        """Sets the dossierid of this PortingPerformed.


        :param dossierid: The dossierid of this PortingPerformed.  # noqa: E501
        :type: str
        """
        if dossierid is None:
            raise ValueError("Invalid value for `dossierid`, must not be `None`")  # noqa: E501
        if dossierid is not None and not re.search(r'[-0-9A-Z]{5,26}', dossierid):  # noqa: E501
            raise ValueError(r"Invalid value for `dossierid`, must be a follow pattern or equal to `/[-0-9A-Z]{5,26}/`")  # noqa: E501

        self._dossierid = dossierid

    @property
    def recipientnetworkoperator(self):
        """Gets the recipientnetworkoperator of this PortingPerformed.  # noqa: E501


        :return: The recipientnetworkoperator of this PortingPerformed.  # noqa: E501
        :rtype: str
        """
        return self._recipientnetworkoperator

    @recipientnetworkoperator.setter
    def recipientnetworkoperator(self, recipientnetworkoperator):
        """Sets the recipientnetworkoperator of this PortingPerformed.


        :param recipientnetworkoperator: The recipientnetworkoperator of this PortingPerformed.  # noqa: E501
        :type: str
        """
        if recipientnetworkoperator is None:
            raise ValueError("Invalid value for `recipientnetworkoperator`, must not be `None`")  # noqa: E501
        if recipientnetworkoperator is not None and not re.search(r'[0-9A-Z]{3,6}', recipientnetworkoperator):  # noqa: E501
            raise ValueError(r"Invalid value for `recipientnetworkoperator`, must be a follow pattern or equal to `/[0-9A-Z]{3,6}/`")  # noqa: E501

        self._recipientnetworkoperator = recipientnetworkoperator

    @property
    def donornetworkoperator(self):
        """Gets the donornetworkoperator of this PortingPerformed.  # noqa: E501


        :return: The donornetworkoperator of this PortingPerformed.  # noqa: E501
        :rtype: str
        """
        return self._donornetworkoperator

    @donornetworkoperator.setter
    def donornetworkoperator(self, donornetworkoperator):
        """Sets the donornetworkoperator of this PortingPerformed.


        :param donornetworkoperator: The donornetworkoperator of this PortingPerformed.  # noqa: E501
        :type: str
        """
        if donornetworkoperator is None:
            raise ValueError("Invalid value for `donornetworkoperator`, must not be `None`")  # noqa: E501
        if donornetworkoperator is not None and not re.search(r'[0-9A-Z]{3,6}', donornetworkoperator):  # noqa: E501
            raise ValueError(r"Invalid value for `donornetworkoperator`, must be a follow pattern or equal to `/[0-9A-Z]{3,6}/`")  # noqa: E501

        self._donornetworkoperator = donornetworkoperator

    @property
    def actualdatetime(self):
        """Gets the actualdatetime of this PortingPerformed.  # noqa: E501


        :return: The actualdatetime of this PortingPerformed.  # noqa: E501
        :rtype: str
        """
        return self._actualdatetime

    @actualdatetime.setter
    def actualdatetime(self, actualdatetime):
        """Sets the actualdatetime of this PortingPerformed.


        :param actualdatetime: The actualdatetime of this PortingPerformed.  # noqa: E501
        :type: str
        """
        if actualdatetime is not None and not re.search(r'[0-9]{14}', actualdatetime):  # noqa: E501
            raise ValueError(r"Invalid value for `actualdatetime`, must be a follow pattern or equal to `/[0-9]{14}/`")  # noqa: E501

        self._actualdatetime = actualdatetime

    @property
    def batchporting(self):
        """Gets the batchporting of this PortingPerformed.  # noqa: E501


        :return: The batchporting of this PortingPerformed.  # noqa: E501
        :rtype: str
        """
        return self._batchporting

    @batchporting.setter
    def batchporting(self, batchporting):
        """Sets the batchporting of this PortingPerformed.


        :param batchporting: The batchporting of this PortingPerformed.  # noqa: E501
        :type: str
        """
        if batchporting is not None and not re.search(r'[Y]{0,}', batchporting):  # noqa: E501
            raise ValueError(r"Invalid value for `batchporting`, must be a follow pattern or equal to `/[Y]{0,}/`")  # noqa: E501

        self._batchporting = batchporting

    @property
    def repeats(self):
        """Gets the repeats of this PortingPerformed.  # noqa: E501


        :return: The repeats of this PortingPerformed.  # noqa: E501
        :rtype: list[PortingPerformedRepeats]
        """
        return self._repeats

    @repeats.setter
    def repeats(self, repeats):
        """Sets the repeats of this PortingPerformed.


        :param repeats: The repeats of this PortingPerformed.  # noqa: E501
        :type: list[PortingPerformedRepeats]
        """
        if repeats is None:
            raise ValueError("Invalid value for `repeats`, must not be `None`")  # noqa: E501

        self._repeats = repeats

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PortingPerformed, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PortingPerformed):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from coin_sdk.number_portability.domain import MessageType
from coin_sdk.number_portability.messages.common import HeaderCreator
from coin_sdk.number_portability.messages.message import Message
from .porting_request_answer_delayed import PortingRequestAnswerDelayed
from .porting_request_answer_delayed_body import PortingRequestAnswerDelayedBody
from .porting_request_answer_delayed_message import PortingRequestAnswerDelayedMessage


class PortingRequestAnswerDelayedBuilder(HeaderCreator):
    def __init__(self):
        HeaderCreator.__init__(self)
        self._dossierid = None
        self._donornetworkoperator = None
        self._donorserviceprovider = None
        self._answerduedatetime = None
        self._reasoncode = None

    def set_dossierid(self, dossierid):
        self._dossierid = dossierid
        return self

    def set_donornetworkoperator(self, donornetworkoperator: str):
        self._donornetworkoperator = donornetworkoperator
        return self

    def set_donorserviceprovider(self, donorserviceprovider: str):
        self._donorserviceprovider = donorserviceprovider
        return self

    def set_answerduedatetime(self, answerduedatetime: str):
        self._answerduedatetime = answerduedatetime
        return self

    def set_reasoncode(self, reasoncode: str):
        self._reasoncode = reasoncode
        return self

    def build(self):
        delayed = PortingRequestAnswerDelayed(
            self._dossierid,
            self._donornetworkoperator,
            self._donorserviceprovider,
            self._answerduedatetime,
            self._reasoncode
        )
        body = PortingRequestAnswerDelayedBody(delayed)
        message = PortingRequestAnswerDelayedMessage(self._header, body)
        return Message(message, MessageType.PORTING_REQUEST_ANSWER_DELAYED_V1)




# coding: utf-8

"""
    Coin CRDB Rest API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 3.0.0
    Contact: devops@coin.nl
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from coin_sdk.number_portability.messages.common.number_series import NumberSeries  # noqa: F401,E501


class PortingRequestAnswerSeq(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'numberseries': 'NumberSeries',
        'blockingcode': 'str',
        'firstpossibledate': 'str',
        'note': 'str',
        'donornetworkoperator': 'str',
        'donorserviceprovider': 'str'
    }

    attribute_map = {
        'numberseries': 'numberseries',
        'blockingcode': 'blockingcode',
        'firstpossibledate': 'firstpossibledate',
        'note': 'note',
        'donornetworkoperator': 'donornetworkoperator',
        'donorserviceprovider': 'donorserviceprovider'
    }

    def __init__(self, numberseries=None, blockingcode=None, firstpossibledate=None, note=None, donornetworkoperator=None, donorserviceprovider=None):  # noqa: E501
        """PortingRequestAnswerSeq - a model defined in Swagger"""  # noqa: E501

        self._numberseries = None
        self._blockingcode = None
        self._firstpossibledate = None
        self._note = None
        self._donornetworkoperator = None
        self._donorserviceprovider = None
        self.discriminator = None

        self.numberseries = numberseries
        if blockingcode is not None:
            self.blockingcode = blockingcode
        if firstpossibledate is not None:
            self.firstpossibledate = firstpossibledate
        if note is not None:
            self.note = note
        if donornetworkoperator is not None:
            self.donornetworkoperator = donornetworkoperator
        if donorserviceprovider is not None:
            self.donorserviceprovider = donorserviceprovider

    @property
    def numberseries(self):
        """Gets the numberseries of this PortingRequestAnswerSeq.  # noqa: E501


        :return: The numberseries of this PortingRequestAnswerSeq.  # noqa: E501
        :rtype: NumberSeries
        """
        return self._numberseries

    @numberseries.setter
    def numberseries(self, numberseries):
        """Sets the numberseries of this PortingRequestAnswerSeq.


        :param numberseries: The numberseries of this PortingRequestAnswerSeq.  # noqa: E501
        :type: NumberSeries
        """
        if numberseries is None:
            raise ValueError("Invalid value for `numberseries`, must not be `None`")  # noqa: E501

        self._numberseries = numberseries

    @property
    def blockingcode(self):
        """Gets the blockingcode of this PortingRequestAnswerSeq.  # noqa: E501


        :return: The blockingcode of this PortingRequestAnswerSeq.  # noqa: E501
        :rtype: str
        """
        return self._blockingcode

    @blockingcode.setter
    def blockingcode(self, blockingcode):
        """Sets the blockingcode of this PortingRequestAnswerSeq.


        :param blockingcode: The blockingcode of this PortingRequestAnswerSeq.  # noqa: E501
        :type: str
        """
        if blockingcode is not None and not re.search(r'[1-9][0-9]{0,4}', blockingcode):  # noqa: E501
            raise ValueError(r"Invalid value for `blockingcode`, must be a follow pattern or equal to `/[1-9][0-9]{0,4}/`")  # noqa: E501

        self._blockingcode = blockingcode

    @property
    def firstpossibledate(self):
        """Gets the firstpossibledate of this PortingRequestAnswerSeq.  # noqa: E501


        :return: The firstpossibledate of this PortingRequestAnswerSeq.  # noqa: E501
        :rtype: str
        """
        return self._firstpossibledate

    @firstpossibledate.setter
    def firstpossibledate(self, firstpossibledate):
        """Sets the firstpossibledate of this PortingRequestAnswerSeq.


        :param firstpossibledate: The firstpossibledate of this PortingRequestAnswerSeq.  # noqa: E501
        :type: str
        """
        if firstpossibledate is not None and not re.search(r'[0-9]{14}', firstpossibledate):  # noqa: E501
            raise ValueError(r"Invalid value for `firstpossibledate`, must be a follow pattern or equal to `/[0-9]{14}/`")  # noqa: E501

        self._firstpossibledate = firstpossibledate

    @property
    def note(self):
        """Gets the note of this PortingRequestAnswerSeq.  # noqa: E501


        :return: The note of this PortingRequestAnswerSeq.  # noqa: E501
        :rtype: str
        """
        return self._note

    @note.setter
    def note(self, note):
        """Sets the note of this PortingRequestAnswerSeq.


        :param note: The note of this PortingRequestAnswerSeq.  # noqa: E501
        :type: str
        """
        if note is not None and len(note) > 100:
            raise ValueError("Invalid value for `note`, length must be less than or equal to `100`")  # noqa: E501
        if note is not None and len(note) < 1:
            raise ValueError("Invalid value for `note`, length must be greater than or equal to `1`")  # noqa: E501

        self._note = note

    @property
    def donornetworkoperator(self):
        """Gets the donornetworkoperator of this PortingRequestAnswerSeq.  # noqa: E501


        :return: The donornetworkoperator of this PortingRequestAnswerSeq.  # noqa: E501
        :rtype: str
        """
        return self._donornetworkoperator

    @donornetworkoperator.setter
    def donornetworkoperator(self, donornetworkoperator):
        """Sets the donornetworkoperator of this PortingRequestAnswerSeq.


        :param donornetworkoperator: The donornetworkoperator of this PortingRequestAnswerSeq.  # noqa: E501
        :type: str
        """
        if donornetworkoperator is not None and not re.search(r'[0-9A-Z]{3,6}', donornetworkoperator):  # noqa: E501
            raise ValueError(r"Invalid value for `donornetworkoperator`, must be a follow pattern or equal to `/[0-9A-Z]{3,6}/`")  # noqa: E501

        self._donornetworkoperator = donornetworkoperator

    @property
    def donorserviceprovider(self):
        """Gets the donorserviceprovider of this PortingRequestAnswerSeq.  # noqa: E501


        :return: The donorserviceprovider of this PortingRequestAnswerSeq.  # noqa: E501
        :rtype: str
        """
        return self._donorserviceprovider

    @donorserviceprovider.setter
    def donorserviceprovider(self, donorserviceprovider):
        """Sets the donorserviceprovider of this PortingRequestAnswerSeq.


        :param donorserviceprovider: The donorserviceprovider of this PortingRequestAnswerSeq.  # noqa: E501
        :type: str
        """
        if donorserviceprovider is not None and not re.search(r'[0-9A-Z]{3,6}', donorserviceprovider):  # noqa: E501
            raise ValueError(r"Invalid value for `donorserviceprovider`, must be a follow pattern or equal to `/[0-9A-Z]{3,6}/`")  # noqa: E501

        self._donorserviceprovider = donorserviceprovider

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PortingRequestAnswerSeq, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PortingRequestAnswerSeq):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from coin_sdk.number_portability.domain import MessageType

class Message:
    def __init__(self, message, message_type: MessageType):
        self._message = message
        self._message_type = message_type

    def get_message(self):
        return self._message

    def get_message_type(self):
        return self._message_type

    def to_dict(self):
        return {'message': self._message.to_dict()}

    def __str__(self):
        return str(self.to_dict())



# coding: utf-8

"""
    Coin CRDB Rest API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 3.0.0
    Contact: devops@coin.nl
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class CustomerInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'lastname': 'str',
        'companyname': 'str',
        'housenr': 'str',
        'housenrext': 'str',
        'postcode': 'str',
        'customerid': 'str'
    }

    attribute_map = {
        'lastname': 'lastname',
        'companyname': 'companyname',
        'housenr': 'housenr',
        'housenrext': 'housenrext',
        'postcode': 'postcode',
        'customerid': 'customerid'
    }

    def __init__(self, lastname=None, companyname=None, housenr=None, housenrext=None, postcode=None, customerid=None):  # noqa: E501
        """CustomerInfo - a model defined in Swagger"""  # noqa: E501

        self._lastname = None
        self._companyname = None
        self._housenr = None
        self._housenrext = None
        self._postcode = None
        self._customerid = None
        self.discriminator = None

        if lastname is not None:
            self.lastname = lastname
        if companyname is not None:
            self.companyname = companyname
        if housenr is not None:
            self.housenr = housenr
        if housenrext is not None:
            self.housenrext = housenrext
        if postcode is not None:
            self.postcode = postcode
        if customerid is not None:
            self.customerid = customerid

    @property
    def lastname(self):
        """Gets the lastname of this CustomerInfo.  # noqa: E501


        :return: The lastname of this CustomerInfo.  # noqa: E501
        :rtype: str
        """
        return self._lastname

    @lastname.setter
    def lastname(self, lastname):
        """Sets the lastname of this CustomerInfo.


        :param lastname: The lastname of this CustomerInfo.  # noqa: E501
        :type: str
        """
        if lastname is not None and len(lastname) > 70:
            raise ValueError("Invalid value for `lastname`, length must be less than or equal to `70`")  # noqa: E501
        if lastname is not None and len(lastname) < 1:
            raise ValueError("Invalid value for `lastname`, length must be greater than or equal to `1`")  # noqa: E501

        self._lastname = lastname

    @property
    def companyname(self):
        """Gets the companyname of this CustomerInfo.  # noqa: E501


        :return: The companyname of this CustomerInfo.  # noqa: E501
        :rtype: str
        """
        return self._companyname

    @companyname.setter
    def companyname(self, companyname):
        """Sets the companyname of this CustomerInfo.


        :param companyname: The companyname of this CustomerInfo.  # noqa: E501
        :type: str
        """
        if companyname is not None and len(companyname) > 70:
            raise ValueError("Invalid value for `companyname`, length must be less than or equal to `70`")  # noqa: E501
        if companyname is not None and len(companyname) < 1:
            raise ValueError("Invalid value for `companyname`, length must be greater than or equal to `1`")  # noqa: E501

        self._companyname = companyname

    @property
    def housenr(self):
        """Gets the housenr of this CustomerInfo.  # noqa: E501


        :return: The housenr of this CustomerInfo.  # noqa: E501
        :rtype: str
        """
        return self._housenr

    @housenr.setter
    def housenr(self, housenr):
        """Sets the housenr of this CustomerInfo.


        :param housenr: The housenr of this CustomerInfo.  # noqa: E501
        :type: str
        """
        if housenr is not None and len(housenr) > 5:
            raise ValueError("Invalid value for `housenr`, length must be less than or equal to `5`")  # noqa: E501
        if housenr is not None and len(housenr) < 1:
            raise ValueError("Invalid value for `housenr`, length must be greater than or equal to `1`")  # noqa: E501

        self._housenr = housenr

    @property
    def housenrext(self):
        """Gets the housenrext of this CustomerInfo.  # noqa: E501


        :return: The housenrext of this CustomerInfo.  # noqa: E501
        :rtype: str
        """
        return self._housenrext

    @housenrext.setter
    def housenrext(self, housenrext):
        """Sets the housenrext of this CustomerInfo.


        :param housenrext: The housenrext of this CustomerInfo.  # noqa: E501
        :type: str
        """
        if housenrext is not None and len(housenrext) > 4:
            raise ValueError("Invalid value for `housenrext`, length must be less than or equal to `4`")  # noqa: E501
        if housenrext is not None and len(housenrext) < 1:
            raise ValueError("Invalid value for `housenrext`, length must be greater than or equal to `1`")  # noqa: E501

        self._housenrext = housenrext

    @property
    def postcode(self):
        """Gets the postcode of this CustomerInfo.  # noqa: E501


        :return: The postcode of this CustomerInfo.  # noqa: E501
        :rtype: str
        """
        return self._postcode

    @postcode.setter
    def postcode(self, postcode):
        """Sets the postcode of this CustomerInfo.


        :param postcode: The postcode of this CustomerInfo.  # noqa: E501
        :type: str
        """
        if postcode is not None and not re.search(r'[0-9A-Z]{1,6}', postcode):  # noqa: E501
            raise ValueError(r"Invalid value for `postcode`, must be a follow pattern or equal to `/[0-9A-Z]{1,6}/`")  # noqa: E501

        self._postcode = postcode

    @property
    def customerid(self):
        """Gets the customerid of this CustomerInfo.  # noqa: E501


        :return: The customerid of this CustomerInfo.  # noqa: E501
        :rtype: str
        """
        return self._customerid

    @customerid.setter
    def customerid(self, customerid):
        """Sets the customerid of this CustomerInfo.


        :param customerid: The customerid of this CustomerInfo.  # noqa: E501
        :type: str
        """
        if customerid is not None and len(customerid) > 20:
            raise ValueError("Invalid value for `customerid`, length must be less than or equal to `20`")  # noqa: E501
        if customerid is not None and len(customerid) < 1:
            raise ValueError("Invalid value for `customerid`, length must be greater than or equal to `1`")  # noqa: E501

        self._customerid = customerid

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CustomerInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CustomerInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import logging
import sys

from coin_sdk.common.config import Config


class NpConfig(Config):
    def __init__(self, base_uri, consumer_name, private_key_file, hmac_secret, number_of_retries=3, backoff_period=1):
        super().__init__(base_uri, consumer_name, private_key_file, hmac_secret)
        self._base_uri = base_uri
        self._consumer_name = consumer_name
        self._private_key_file = private_key_file
        self._hmac_secret = hmac_secret
        self._number_of_retries = number_of_retries
        self._backoff_period = backoff_period

    @property
    def base_uri(self):
        return self._base_uri

    @property
    def api_url(self):
        return self._base_uri + '/number-portability/v1/dossiers'

    @property
    def sse_url(self):
        return self.api_url + '/events'

    @property
    def consumer_name(self):
        return self._consumer_name

    @property
    def private_key_file(self):
        return self._private_key_file

    @property
    def hmac_secret(self):
        return self._hmac_secret

    @property
    def number_of_retries(self):
        return self._number_of_retries

    @property
    def backoff_period(self):
        return self._backoff_period


_format = '%(asctime)s - %(name)-8s - %(levelname)s - %(message)s'


def set_logging(format=_format, level=logging.ERROR, stream=sys.stdout):
    logging.basicConfig(format=format, level=level, stream=stream)



from datetime import datetime

from .header import Header
from .sender import Sender
from .receiver import Receiver


class HeaderCreator:
    def __init__(self):
        self._header = None

    def set_header(self, sender_network_operator: str, receiver_network_operator: str, sender_service_provider: str = None, receiver_service_provider: str = None):
        sender = Sender(sender_network_operator, sender_service_provider)
        receiver = Receiver(receiver_network_operator, receiver_service_provider)
        timestamp = datetime.now().strftime('%Y%m%d%H%M%S')
        self._header = Header(receiver, sender, timestamp)
        return self



# coding: utf-8

"""
    Coin CRDB Rest API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 3.0.0
    Contact: devops@coin.nl
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from coin_sdk.number_portability.messages.common import EnumRepeats, NumberSeries # noqa: F401,E501


class PortingPerformedSeq(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'numberseries': 'NumberSeries',
        'backporting': 'str',
        'pop': 'str',
        'repeats': 'list[EnumRepeats]'
    }

    attribute_map = {
        'numberseries': 'numberseries',
        'backporting': 'backporting',
        'pop': 'pop',
        'repeats': 'repeats'
    }

    def __init__(self, numberseries=None, backporting=None, pop=None, repeats=None):  # noqa: E501
        """PortingPerformedSeq - a model defined in Swagger"""  # noqa: E501

        self._numberseries = None
        self._backporting = None
        self._pop = None
        self._repeats = None
        self.discriminator = None

        self.numberseries = numberseries
        if backporting is not None:
            self.backporting = backporting
        if pop is not None:
            self.pop = pop
        if repeats is not None:
            self.repeats = repeats

    @property
    def numberseries(self):
        """Gets the numberseries of this PortingPerformedSeq.  # noqa: E501


        :return: The numberseries of this PortingPerformedSeq.  # noqa: E501
        :rtype: NumberSeries
        """
        return self._numberseries

    @numberseries.setter
    def numberseries(self, numberseries):
        """Sets the numberseries of this PortingPerformedSeq.


        :param numberseries: The numberseries of this PortingPerformedSeq.  # noqa: E501
        :type: NumberSeries
        """
        if numberseries is None:
            raise ValueError("Invalid value for `numberseries`, must not be `None`")  # noqa: E501

        self._numberseries = numberseries

    @property
    def backporting(self):
        """Gets the backporting of this PortingPerformedSeq.  # noqa: E501


        :return: The backporting of this PortingPerformedSeq.  # noqa: E501
        :rtype: str
        """
        return self._backporting

    @backporting.setter
    def backporting(self, backporting):
        """Sets the backporting of this PortingPerformedSeq.


        :param backporting: The backporting of this PortingPerformedSeq.  # noqa: E501
        :type: str
        """
        if backporting is not None and not re.search(r'[Y]{0,}', backporting):  # noqa: E501
            raise ValueError(r"Invalid value for `backporting`, must be a follow pattern or equal to `/[Y]{0,}/`")  # noqa: E501

        self._backporting = backporting

    @property
    def pop(self):
        """Gets the pop of this PortingPerformedSeq.  # noqa: E501


        :return: The pop of this PortingPerformedSeq.  # noqa: E501
        :rtype: str
        """
        return self._pop

    @pop.setter
    def pop(self, pop):
        """Sets the pop of this PortingPerformedSeq.


        :param pop: The pop of this PortingPerformedSeq.  # noqa: E501
        :type: str
        """
        if pop is not None and len(pop) > 24:
            raise ValueError("Invalid value for `pop`, length must be less than or equal to `24`")  # noqa: E501
        if pop is not None and len(pop) < 1:
            raise ValueError("Invalid value for `pop`, length must be greater than or equal to `1`")  # noqa: E501

        self._pop = pop

    @property
    def repeats(self):
        """Gets the repeats of this PortingPerformedSeq.  # noqa: E501


        :return: The repeats of this PortingPerformedSeq.  # noqa: E501
        :rtype: list[EnumRepeats]
        """
        return self._repeats

    @repeats.setter
    def repeats(self, repeats):
        """Sets the repeats of this PortingPerformedSeq.


        :param repeats: The repeats of this PortingPerformedSeq.  # noqa: E501
        :type: list[EnumRepeats]
        """

        self._repeats = repeats

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PortingPerformedSeq, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PortingPerformedSeq):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from .confirmation_message import ConfirmationMessage
from .customer_info import CustomerInfo
from .enum_profile_seq import EnumProfileSeq
from .enum_repeats import EnumRepeats
from .header import Header
from .header_creator import HeaderCreator
from .number_series import NumberSeries
from .receiver import Receiver
from .sender import Sender
from .enum_repeats_builder import EnumRepeatsBuilder
from .error_message import ErrorMessage



"""
CLI for obtaining JWT access tokens using the PKCE flow
"""

import sys
if sys.version_info < (3,6):
    sys.exit('Sorry, Python < 3.6 is not supported')

from setuptools import find_packages, setup

dependencies = ['requests',
                'cryptography',
                'pyjwt',
                'colorama',
                'sseclient-py']

from distutils.core import setup

setup(
    name='coin_sdk',  # How you named your package folder (MyLib)
    packages=find_packages(exclude=['tests']),
    version="0.2.1",
    license='MIT',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='SDKs for Vereninging COIN\'s apis',  # Give a short description about your library
    author='Vereniging COIN',  # Type in your name
    author_email='devops@coin.nl',  # Type in your E-Mail
    url='https://gitlab.com/verenigingcoin-public/coin-sdk-python',
    # Provide either the link to your github or to your website
    keywords=['COIN', 'Numberportability', 'TELECOM', 'SDK'],  # Keywords that define your package best
    install_requires=dependencies,
    tests_require=dependencies,
    python_requires='>=3.6',
    zip_safe=False,
    platforms='any',
    classifiers=[
        'Development Status :: 4 - Beta',
        # Choose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3.6',
    ],
)



from coin_sdk.number_portability.domain import MessageType
from coin_sdk.number_portability.messages.common import EnumRepeatsBuilder, HeaderCreator
from coin_sdk.number_portability.messages.message import Message
from .porting_performed import PortingPerformed
from .porting_performed_body import PortingPerformedBody
from .porting_performed_message import PortingPerformedMessage
from .porting_performed_repeats import PortingPerformedRepeats
from .porting_performed_seq import PortingPerformedSeq


class PortingPerformedBuilder(HeaderCreator):
    def __init__(self):
        super().__init__()
        self._dossierid = None
        self._recipientnetworkoperator = None
        self._donornetworkoperator = None
        self._actualdatetime = None
        self._batchporting = None
        self._porting_performed_seqs = []

    def set_dossierid(self, dossier_id: str):
        self._dossierid = dossier_id
        return self

    def set_recipientnetworkoperator(self, recipientnetworkoperator: str):
        self._recipientnetworkoperator = recipientnetworkoperator
        return self

    def set_donornetworkoperator(self, donornetworkoperator: str):
        self._donornetworkoperator = donornetworkoperator
        return self

    def set_actualdatetime(self, actualdatetime: str):
        self._actualdatetime = actualdatetime
        return self

    def set_batchporting(self, batchporting: str):
        self._batchporting = batchporting
        return self

    def add_porting_performed_seq(self):
        return PortingPerformedRepeatsBuilder(self)

    def build(self):
        repeats = [PortingPerformedRepeats(seq) for seq in self._porting_performed_seqs]
        porting_performed = PortingPerformed(
            self._dossierid,
            self._recipientnetworkoperator,
            self._donornetworkoperator,
            self._actualdatetime,
            self._batchporting,
            repeats
        )
        body = PortingPerformedBody(porting_performed)
        message = PortingPerformedMessage(self._header, body)
        return Message(message, MessageType.PORTING_PERFORMED_V1)


class PortingPerformedRepeatsBuilder(EnumRepeatsBuilder):
    def __init__(self, porting_performed_builder: PortingPerformedBuilder):
        super().__init__()
        self._porting_performed_builder = porting_performed_builder
        self._backporting = None
        self._pop = None

    def set_number_series(self, start: str, end: str):
        self._set_number_series(start, end)
        return self

    def add_enum_profiles(self, *profileids):
        self._add_enum_prof_seq(*profileids)
        return self

    def set_backporting(self, backporting: str):
        self._backporting = backporting
        return self

    def set_pop(self, pop: str):
        self._pop = pop
        return self

    def finish(self):
        seq = PortingPerformedSeq(
            self._number_series,
            self._backporting,
            self._pop,
            self._enum_repeats
        )
        self._porting_performed_builder._porting_performed_seqs.append(seq)
        return self._porting_performed_builder



from coin_sdk.number_portability.domain import MessageType
from coin_sdk.number_portability.messages.common import CustomerInfo, EnumRepeatsBuilder, HeaderCreator
from coin_sdk.number_portability.messages.message import Message
from .porting_request import PortingRequest
from .porting_request_body import PortingRequestBody
from .porting_request_message import PortingRequestMessage
from .porting_request_repeats import PortingRequestRepeats
from .porting_request_seq import PortingRequestSeq


class PortingRequestBuilder(HeaderCreator):
    def __init__(self):
        super().__init__()
        self._dossierid = None
        self._recipientnetworkoperator = None
        self._recipientserviceprovider = None
        self._donornetworkoperator = None
        self._donorserviceprovider = None
        self._note = None
        self._customerinfo = None
        self._porting_request_seqs = []

    def set_dossierid(self, dossierid: str):
        self._dossierid = dossierid
        return self

    def set_recipientnetworkoperator(self, recipientnetworkoperator: str):
        self._recipientnetworkoperator = recipientnetworkoperator
        return self

    def set_recipientserviceprovider(self, recipientserviceprovider: str):
        self._recipientserviceprovider = recipientserviceprovider
        return self

    def set_donornetworkoperator(self, donornetworkoperator: str):
        self._donornetworkoperator = donornetworkoperator
        return self

    def set_donorserviceprovider(self, donorserviceprovider: str):
        self._donorserviceprovider = donorserviceprovider
        return self

    def set_note(self, note: str):
        self._note = note
        return self

    def set_customerinfo(self, lastname: str = None, companyname: str = None, housenr: str = None, housenrext: str = None, postcode: str = None, customerid: str = None):
        self._customerinfo = CustomerInfo(lastname, companyname, housenr, housenrext, postcode, customerid)
        return self

    def add_porting_request_seq(self):
        return PortingRequestRepeatsBuilder(self)

    def build(self):
        repeats = [PortingRequestRepeats(seq) for seq in self._porting_request_seqs]
        porting_request = PortingRequest(
            self._dossierid,
            self._recipientserviceprovider,
            self._recipientnetworkoperator,
            self._donornetworkoperator,
            self._donorserviceprovider,
            self._customerinfo,
            self._note,
            repeats
        )
        body = PortingRequestBody(porting_request)
        message = PortingRequestMessage(self._header, body)
        return Message(message, MessageType.PORTING_REQUEST_V1)


class PortingRequestRepeatsBuilder(EnumRepeatsBuilder):
    def __init__(self, porting_request_builder: PortingRequestBuilder):
        super().__init__()
        self._porting_request_builder = porting_request_builder

    def set_number_series(self, start: str, end: str):
        self._set_number_series(start, end)
        return self

    def add_enum_profiles(self, *profileids):
        self._add_enum_prof_seq(*profileids)
        return self

    def finish(self):
        seq = PortingRequestSeq(self._number_series, self._enum_repeats)
        self._porting_request_builder._porting_request_seqs.append(seq)
        return self._porting_request_builder



import hashlib
import hmac
import json
import time
import logging
from base64 import b64decode, b64encode
from datetime import datetime

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.serialization import load_pem_private_key
from jwt import PyJWT

from coin_sdk.common.config import Config


class SecurityService:

    def __init__(self, config: Config):
        self._config = config
        self.jwt = PyJWT()
        self.expires_after = 30
        self.not_before = 30
        self.load_private_key()
        self.load_shared_key(self.private_key)
        self.generate_token()

    def load_private_key(self, filename=None):
        filename = filename or self._config.private_key_file
        with open(filename, 'rb') as f:
            private_key_pem = f.read()
        self.private_key = load_pem_private_key(
            private_key_pem, password=None, backend=default_backend())

    def load_shared_key(self, private_key,
                        filename=None):
        filename = filename or self._config.hmac_secret
        encrypted_key = None
        with open(filename, 'rb') as f:
            encrypted_key = b64decode(f.read())
        self.shared_key = private_key.decrypt(encrypted_key, padding.PKCS1v15())

    def generate_token(self):
        self.now = int(time.mktime(datetime.now().timetuple()))
        self.expires_at = self.now + self.expires_after
        message = {
            'iss': self._config.consumer_name,
            'nbf': self.now - self.not_before,
            'exp': self.now + self.expires_after
        }
        token = self.jwt.encode(message, self.private_key, 'RS256')
        self.token = token.decode('ascii')

    def token_has_expired(self):
        now = int(time.mktime(datetime.now().timetuple()))
        return self.token is None or now + 5 > self.now

    def get_token(self):
        if self.token_has_expired():
            self.generate_token()
        return self.token

    def httpdate(self):
        """Return a string representation of a date according to RFC 1123
        (HTTP/1.1).

        The supplied date must be in UTC.
        """
        now = datetime.utcnow()
        weekday = [
            'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'][
            now.weekday()]
        month = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep',
                 'Oct', 'Nov', 'Dec'][now.month - 1]
        return '{}, {:02d} {} {:04d} {:02d}:{:02d}:{:02d} GMT'.format(
            weekday, now.day, month, now.year, now.hour, now.minute, now.second)

    def calculate_digest(self, msg):
        sha = hashlib.sha256()
        sha.update(msg)
        return b64encode(sha.digest())

    def create_digest(self, msg=b''):
        return f'SHA-256={self.calculate_digest(msg).decode("ascii")}'

    def generate_headers(self, url, method='get', request=None):
        uri = '/' + '/'.join(url.split('/')[3::]).split('?')[0]
        request_line = f'{method.upper()} {uri} HTTP/1.1'
        x_date = self.httpdate()
        digest = self.create_digest(json.dumps(request).encode() if request else b'')
        message = bytearray(f'digest: {digest}\nx-date: {x_date}\n{request_line}', 'ascii')
        logging.debug(self.shared_key)
        signature = hmac.new(self.shared_key, msg=message, digestmod=hashlib.sha256).digest()
        authorization = f'hmac username="{self._config.consumer_name}", algorithm="hmac-sha256", ' \
            f'headers="digest x-date request-line", signature="{b64encode(signature).decode()}" '

        return {'digest': digest, 'x-date': x_date, 'authorization': str(authorization)}

    def generate_jwt(self):
        return {'jwt': f'{self.get_token()}'}



import random
import unittest
import logging
import sys
import os
from collections import namedtuple

from requests import HTTPError

from coin_sdk.number_portability.npconfig import NpConfig, set_logging
from coin_sdk.number_portability.domain import ConfirmationStatus
from coin_sdk.number_portability.messages.cancel import CancelBuilder
from coin_sdk.number_portability.messages.deactivation import DeactivationBuilder
from coin_sdk.number_portability.messages.portingperformed import PortingPerformedBuilder
from coin_sdk.number_portability.messages.portingrequest import PortingRequestBuilder
from coin_sdk.number_portability.messages.portingrequestanswer import PortingRequestAnswerBuilder
from coin_sdk.number_portability.messages.portingrequestanswerdelayed import PortingRequestAnswerDelayedBuilder
from coin_sdk.number_portability.receiver import Receiver, OffsetPersister
from coin_sdk.number_portability.sender import Sender

config = NpConfig(
    os.getenv('CRDB_REST_BACKEND', 'http://0.0.0.0:8000'),
    'loadtest-loada',
    private_key_file='./test/setup/private-key.pem',
    hmac_secret='./test/setup/sharedkey.encrypted'
)

sender = Sender(config)
#set_logging(level=logging.DEBUG, stream=sys.stderr)

class ExamplesTest(unittest.TestCase):
    def test_porting_request(self):
        dossier_id = self._generate_random_dossier_id('LOADA')
        porting_request = (
            PortingRequestBuilder()
            .set_dossierid(dossier_id)
            .set_recipientnetworkoperator('LOADB')
            .set_header(sender_network_operator='LOADA', receiver_network_operator='LOADB')
            .add_porting_request_seq()
                .set_number_series('0612345678', '0612345678')
                .finish()
            .add_porting_request_seq()
                .set_number_series('0612345678', '0612345678')
                .add_enum_profiles('PROF1', 'PROF2')
                .finish()
            .set_customerinfo("test", "test bv", "1", "a", "1234AB", "1")
            .build()
        )
        sender.send_message(porting_request)

    def test_cancel(self):
        dossier_id = self._generate_random_dossier_id('LOADA')
        cancel = CancelBuilder() \
            .set_dossierid(dossier_id) \
            .set_header(sender_network_operator='LOADA', receiver_network_operator='LOADB') \
            .build()
        sender.send_message(cancel)

    def test_deactivation(self):
        dossier_id = self._generate_random_dossier_id('LOADA')
        deactivation = (
            DeactivationBuilder()
            .set_header(sender_network_operator='LOADA', receiver_network_operator='LOADB')
            .set_dossierid(dossier_id)
            .set_currentnetworkoperator('LOADB')
            .set_originalnetworkoperator('LOADA')
            .add_deactivation_seq()
                .set_number_series('0612345678', '0612345678')
                .finish()
            .build()
        )
        sender.send_message(deactivation)

    def test_porting_performed(self):
        dossier_id = self._generate_random_dossier_id('LOADA')
        porting_performed = (
            PortingPerformedBuilder()
            .set_header('LOADA', 'LOADB')
            .set_dossierid(dossier_id)
            .set_donornetworkoperator('LOADB')
            .set_recipientnetworkoperator('LOADA')
            .add_porting_performed_seq()
                .set_number_series('0612345678', '0612345678')
                .finish()
            .add_porting_performed_seq()
                .set_number_series('0612345678', '0612345678')
                .add_enum_profiles('PROF1', 'PROF2')
                .finish()
            .build()
        )
        sender.send_message(porting_performed)

    def test_porting_request_answer(self):
        dossier_id = self._generate_random_dossier_id('LOADA')
        porting_performed_answer = (
            PortingRequestAnswerBuilder()
            .set_header('LOADA', 'LOADB')
            .set_dossierid(dossier_id)
            .set_blocking('N')
            .add_porting_request_answer_seq()
                .set_donornetworkoperator('LOADA')
                .set_donorserviceprovider('LOADA')
                .set_firstpossibledate('20190101120000')
                .set_number_series('0612345678', '0612345678')
                .set_note('This is a note')
                .set_blockingcode('99')
                .finish()
            .add_porting_request_answer_seq()
                .set_donornetworkoperator('LOADA')
                .set_donorserviceprovider('LOADA')
                .set_firstpossibledate('20190101120000')
                .set_number_series('0612345678', '0612345678')
                .set_note('This is a note')
                .set_blockingcode('99')
                .finish()
            .build()
        )
        sender.send_message(porting_performed_answer)

    def test_porting_request_answer_delayed(self):
        dossier_id = self._generate_random_dossier_id('LOADA')
        porting_request_answer_delayed = (
            PortingRequestAnswerDelayedBuilder()
            .set_header('LOADA', 'LOADB')
            .set_dossierid(dossier_id)
            .set_donornetworkoperator('LOADB')
            .build()
        )
        sender.send_message(porting_request_answer_delayed)

    def test_send_error(self):
        dossier_id = self._generate_random_dossier_id('LOADA')
        porting_request_answer_delayed = (
            PortingRequestAnswerDelayedBuilder()
                .set_header('LOADA', 'LOADA')
                .set_dossierid(dossier_id)
                .set_donornetworkoperator('LOADB')
                .build()
        )
        try:
            sender.send_message(porting_request_answer_delayed)
        except HTTPError as e:
            print(e)

    def test_receive_message(self):
        TestReceiver(config).start_stream(confirmation_status=ConfirmationStatus.ALL, offset_persister=TestOffsetPersister)

    @staticmethod
    def _generate_random_dossier_id(operator: str):
        random_int = random.randint(10000, 99999)
        return f'{operator}-{random_int}'


class TestReceiver(Receiver):
    def on_keep_alive(self):
        pass

    def on_porting_request(self, message_id, message):
        print('porting request')
        self.handle_message(message_id, message)

    def on_porting_request_answer(self, message_id, message):
        print('porting request answer')
        self.handle_message(message_id, message)

    def on_porting_request_answer_delayed(self, message_id, message):
        print('porting request answer delayed')
        self.handle_message(message_id, message)

    def on_porting_performed(self, message_id, message):
        print('porting performed')
        self.handle_message(message_id, message)

    def on_deactivation(self, message_id, message):
        print('deactivation')
        self.handle_message(message_id, message)

    def on_cancel(self, message_id, message):
        print('cancel')
        self.handle_message(message_id, message)

    def on_error_found(self, message_id, message):
        print('error!')
        self.handle_message(message_id, message)
        self.stop()

    def handle_message(self, message_id, message):
        print(message)
        sender.confirm(message_id)


class TestOffsetPersister(OffsetPersister):
    def __init__(self):
        self._offset = -1

    def get_persisted_offset(self):
        return self._offset

    def persist_offset(self, offset):
        self._offset = offset


if __name__ == '__main__':
    unittest.main()



from coin_sdk.number_portability.domain import MessageType
from coin_sdk.number_portability.messages.common import HeaderCreator
from coin_sdk.number_portability.messages.message import Message
from .cancel import Cancel
from .cancel_message import CancelMessage
from .cancel_body import CancelBody


class CancelBuilder(HeaderCreator):
    def __init__(self):
        HeaderCreator.__init__(self)
        self._dossierid = None
        self._note = None

    def set_dossierid(self, dossierid):
        self._dossierid = dossierid
        return self

    def set_note(self, note: str):
        self._note = note
        return self

    def build(self):
        cancel = Cancel(self._dossierid, self._note)
        body = CancelBody(cancel)
        message = CancelMessage(self._header, body)
        return Message(message, MessageType.CANCEL_V1)

